@@ -3,6 +3,7 @@
 
 #include <stdbool.h>
 #include "global.h"
+#include "libip.h"
 
 #define TIME_IN_CACHE     MINUTE
 #define MAX_CACHE_TIMER     HOUR
@@ -14,7 +15,7 @@ typedef struct type_cached_object {
 	int           timer;
 	time_t        last_changed;
 	volatile int  in_use;
-	t_ipv4        last_ip;
+	t_ip_addr     last_ip;
 
 	struct type_cached_object *prev;
 	struct type_cached_object *next;
