@@ -12,7 +12,6 @@
 #include <errno.h>
 #include <pthread.h>
 #include <sys/socket.h>
-#include <arpa/inet.h>
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
@@ -26,7 +25,7 @@
 #include "log.h"
 
 #define MAX_CGI_DELAY_TIMER MINUTE
-#define DUMMY_BUFFER_SIZE 65536
+#define DUMMY_BUFFER_SIZE 64 * KILOBYTE
 
 static int delay_timer = 0;
 
@@ -47,11 +46,11 @@ void init_load_balancer(t_fcgi_server *fcgi_server) {
 	}
 }
 
-t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ipv4 client_ip) {
+t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ip_addr *client_ip) {
 	t_connect_to  *connect_to = NULL;
 	t_cgi_session *cgi_session;
 	bool search_new_fcgi_server = true;
-	int i;
+	unsigned char i;
 
 	/* Only one connect_to?
 	 */
@@ -59,20 +58,21 @@ t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ipv4 client_ip) {
 		return fcgi_server->connect_to;
 	}
 
-	i = ip_index(client_ip);
+	i = index_by_ip(client_ip);
 	pthread_mutex_lock(&fcgi_server->cgi_session_mutex[i]);
 
 	/* Search in cgi_session_list
 	 */
 	cgi_session = fcgi_server->cgi_session_list[i];
 	while (cgi_session != NULL) {
-		if (cgi_session->client_ip == client_ip) {
+		if (same_ip(&(cgi_session->client_ip), client_ip)) {
 			if (cgi_session->timer - delay_timer > 0) {
 				if (cgi_session->connect_to->available) {
 					cgi_session->timer = fcgi_server->session_timeout + delay_timer;
 					connect_to = cgi_session->connect_to;
 				} else {
-					cgi_session->client_ip = 0;
+					/* Make sure it won't match the next round */
+					cgi_session->client_ip.family = AF_UNSPEC;
 				}
 				search_new_fcgi_server = false;
 			}
@@ -94,7 +94,7 @@ t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ipv4 client_ip) {
 		 */
 		if (fcgi_server->session_timeout > 0) {
 			if ((cgi_session = (t_cgi_session*)malloc(sizeof(t_cgi_session))) != NULL) {
-				cgi_session->client_ip = client_ip;
+				copy_ip(&(cgi_session->client_ip), client_ip);
 				cgi_session->connect_to = connect_to;
 				cgi_session->timer = fcgi_server->session_timeout + delay_timer;
 
@@ -406,27 +406,41 @@ t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
  *  FastCGI servers
  *  ================
  */
-
 int connect_to_fcgi_server(t_connect_to *connect_to) {
-	int sock;
-	struct hostent *hostinfo;
-	struct sockaddr_in saddr;
+	int sock = -1;
+	struct sockaddr_in saddr4;
+#ifdef HAVE_IPV6
+	struct sockaddr_in6 saddr6;
+#endif
 
-	if ((hostinfo = gethostbyname(connect_to->host)) != NULL) {
+	if (connect_to->host.family == AF_INET) {
+		/* IPv4
+		 */
 		if ((sock = socket(AF_INET, SOCK_STREAM, 0)) > 0) {
-			memset(&saddr, 0, sizeof(struct sockaddr_in));
-			saddr.sin_family = AF_INET;
-			saddr.sin_port = htons(connect_to->port);
-			memcpy(&saddr.sin_addr.s_addr, hostinfo->h_addr, 4);
-			if (connect(sock, (struct sockaddr*)&saddr, sizeof(struct sockaddr_in)) != 0) {
+			memset(&saddr4, 0, sizeof(struct sockaddr_in));
+			saddr4.sin_family = AF_INET;
+			saddr4.sin_port = htons(connect_to->port);
+			memcpy(&saddr4.sin_addr.s_addr, &(connect_to->host.value), connect_to->host.size);
+			if (connect(sock, (struct sockaddr*)&saddr4, sizeof(struct sockaddr_in)) != 0) {
 				close(sock);
 				sock = -1;
 			}
-		} else {
-			sock = -1;
 		}
-	} else {
-		sock = -1;
+#ifdef HAVE_IPV6
+	} else if (connect_to->host.family == AF_INET6) {
+		/* IPv6
+		 */
+		if ((sock = socket(AF_INET6, SOCK_STREAM, 0)) > 0) {
+			memset(&saddr6, 0, sizeof(struct sockaddr_in6));
+			saddr6.sin6_family = AF_INET6;
+			saddr6.sin6_port = htons(connect_to->port);
+			memcpy(&saddr6.sin6_addr.s6_addr, &(connect_to->host.value), connect_to->host.size);
+			if (connect(sock, (struct sockaddr*)&saddr6, sizeof(struct sockaddr_in6)) != 0) {
+				close(sock);
+				sock = -1;
+			}
+		}
+#endif
 	}
 
 	return sock;
