@@ -33,7 +33,7 @@ void increment_transfer(int counter, long bytes);
 void init_command_module(void);
 int add_admin(int sock);
 void remove_admin(int sock);
-void check_adminlist(void);
+void check_admin_list(void);
 t_admin *first_admin(void);
 t_admin *next_admin(void);
 int handle_admin(t_admin *admin, t_config *config);
