@@ -5,6 +5,7 @@
 #include <string.h>
 #include "alternative.h"
 #include "envir.h"
+#include "libip.h"
 #include "session.h"
 
 #define SSL_VAR_SIZE 512
@@ -88,9 +89,8 @@ static int headerfield_to_environment(t_session *session, t_fcgi_buffer *fcgi_bu
 /* Set environment variables for CGI script.
  */
 void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
-	char ip[20], len[20], value[10], *name;
+	char ip[MAX_IP_STR_LEN], len[20], value[10], *name;
 	size_t len1, len2;
-	unsigned char *ip_address;
 	t_keyvalue *envir;
 #ifdef HAVE_SSL
 	char subject[SSL_VAR_SIZE + 1], issuer[SSL_VAR_SIZE + 1];
@@ -131,9 +131,9 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 #endif
 		add_to_environment_chroot(session, fcgi_buffer, "DOCUMENT_ROOT", session->host->website_root);
 
-	ip_address = (unsigned char*)&(session->ip_address);
-	ip[19] = '\0';
-	snprintf(ip, 19, "%hhu.%hhu.%hhu.%hhu", ip_address[0], ip_address[1], ip_address[2], ip_address[3]);
+	if (inet_ntop(session->ip_address.family, &(session->ip_address.value), ip, MAX_IP_STR_LEN) == NULL) {
+		strcpy(ip, "?.?.?.?");
+	}
 	add_to_environment(fcgi_buffer, "REMOTE_ADDR", ip);
 
 	if (session->vars != NULL) {
