@@ -52,7 +52,7 @@
 #include "rewrite.h"
 #endif
 
-#define REQUEST_BUFFER_CHUNK  4096
+#define REQUEST_BUFFER_CHUNK     4 * KILOBYTE
 #define MAIN_LOOP_TIMER          1
 #define PTHREAD_STACK_SIZE     512 * KILOBYTE
 
@@ -84,21 +84,24 @@ char *fb_symlink     = "symlink not allowed";
 char *fb_accesslist  = "access denied via accesslist";
 char *fb_alterlist   = "access denied via alterlist";
 char *version_string = "Hiawatha v"VERSION
-#ifdef HAVE_SSL
-			", SSL"
-#endif
 #ifdef HAVE_CACHE
 			", cache"
 #endif
-#ifdef HAVE_REWRITE
-			", URL rewrite"
-#endif
 #ifdef HAVE_COMMAND
 			", CommandChannel"
 #endif
 #ifdef DEBUG
 			", debug"
 #endif
+#ifdef HAVE_IPV6
+			", IPv6"
+#endif
+#ifdef HAVE_SSL
+			", SSL"
+#endif
+#ifdef HAVE_REWRITE
+			", URL rewrite"
+#endif
 ;
 
 #ifdef DEBUG
@@ -750,16 +753,16 @@ int uri_to_path(t_session *session) {
 
 t_access allow_client(t_session *session) {
 	char *x_forwarded_for;
-	t_ipv4 forwarded_ip;
+	t_ip_addr forwarded_ip;
 	t_access access;
 
-	if ((access = ip_allowed(session->ip_address, session->host->access_list)) != allow) {
+	if ((access = ip_allowed(&(session->ip_address), session->host->access_list)) != allow) {
 		return access;
 	} else if ((x_forwarded_for = get_headerfield("X-Forwarded-For:", session->headerfields)) == NULL) {
 		return allow;
 	} else if (parse_ip(x_forwarded_for, &forwarded_ip) == -1) {
 		return allow;
-	} else if (ip_allowed(forwarded_ip, session->host->access_list) == deny) {
+	} else if (ip_allowed(&forwarded_ip, session->host->access_list) == deny) {
 		return deny;
 	}
 
@@ -826,6 +829,7 @@ int serve_client(t_session *session) {
 	char *search, *conffile, *qmark;
 	t_host *host_record;
 	bool file_not_found = false;
+	t_access access;
 	t_fsbool is_dir;
 #ifdef HAVE_REWRITE
 	int i;
@@ -954,7 +958,7 @@ int serve_client(t_session *session) {
 		return result;
 	}
 
-	switch (allow_client(session)) {
+	switch (access = allow_client(session)) {
 		case deny:
 			log_error(session, fb_accesslist);
 			return 403;
@@ -962,7 +966,7 @@ int serve_client(t_session *session) {
 			break;
 		case pwd:
 		case unknown:
-			if (http_authentication_oke(session) == false) {
+			if (http_authentication_oke(session, access == unknown) == false) {
 				return 401;
 			}
 	}
@@ -1042,16 +1046,16 @@ void handle_request_result(t_session *session, int result) {
 		case fe_MAX_REQUESTSIZE:
 			log_system(session, "Maximum request size reached");
 			send_code(session, 413);
-			if ((session->config->ban_on_max_request_size > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
-				ban_ip(session->ip_address, session->config->ban_on_max_request_size, session->config->kick_on_ban);
+			if ((session->config->ban_on_max_request_size > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+				ban_ip(&(session->ip_address), session->config->ban_on_max_request_size, session->config->kick_on_ban);
 				log_system(session, "Client banned because of sending a too large request");
 			}
 			break;
 		case fe_TIMEOUT:
 			if (session->kept_alive == 0) {
 				send_code(session, 408);
-				if ((session->config->ban_on_timeout > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
-					ban_ip(session->ip_address, session->config->ban_on_timeout, session->config->kick_on_ban);
+				if ((session->config->ban_on_timeout > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+					ban_ip(&(session->ip_address), session->config->ban_on_timeout, session->config->kick_on_ban);
 					log_system(session, "Client banned because of connection timeout");
 				} else {
 					log_system(session, "Timeout while waiting for request");
@@ -1076,16 +1080,16 @@ void handle_request_result(t_session *session, int result) {
 			break;
 		case rr_CMD_INJECTION:
 			log_request(session, result);
-			if (ip_allowed(session->ip_address, session->config->banlist_mask)) {
-				ban_ip(session->ip_address, session->config->ban_on_cmdi, session->config->kick_on_ban);
+			if (ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+				ban_ip(&(session->ip_address), session->config->ban_on_cmdi, session->config->kick_on_ban);
 				log_system(session, "Client banned because of command injection");
 			}
 			send_code(session, 400);
 			break;
 		case rr_SQL_INJECTION:
 			log_request(session, result);
-			if (ip_allowed(session->ip_address, session->config->banlist_mask)) {
-				ban_ip(session->ip_address, session->config->ban_on_sqli, session->config->kick_on_ban);
+			if (ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+				ban_ip(&(session->ip_address), session->config->ban_on_sqli, session->config->kick_on_ban);
 				log_system(session, "Client banned because of SQL injection");
 			}
 			send_code(session, 400);
@@ -1116,8 +1120,8 @@ void handle_request_result(t_session *session, int result) {
 					session->keep_alive = false;
 				}
 			}
-			if ((session->config->ban_on_garbage > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
-				ban_ip(session->ip_address, session->config->ban_on_garbage, session->config->kick_on_ban);
+			if ((session->config->ban_on_garbage > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+				ban_ip(&(session->ip_address), session->config->ban_on_garbage, session->config->kick_on_ban);
 				log_system(session, "Client banned because of sending garbage");
 			}
 			break;
@@ -1173,8 +1177,8 @@ void connection_handler(t_session *session) {
 
 			if (session->keep_alive && (session->config->ban_on_flooding > 0)) {
 				if (client_is_flooding(session)) {
-					if (ip_allowed(session->ip_address, session->config->banlist_mask)) {
-						ban_ip(session->ip_address, session->config->ban_on_flooding, session->config->kick_on_ban);
+					if (ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+						ban_ip(&(session->ip_address), session->config->ban_on_flooding, session->config->kick_on_ban);
 						log_system(session, "Client banned because of flooding");
 						session->keep_alive = false;
 					}
@@ -1259,21 +1263,24 @@ int fill_read_fds(fd_set *read_fds, t_binding *binding, t_binding *bind_command)
 /* Create a socketlist.
  */
 int bind_sockets(t_binding *binding) {
-	int one;
-	struct sockaddr_in saddr;
-	char *ip_address;
+	char ip_address[MAX_IP_STR_LEN], separator;
+	struct sockaddr_in  saddr4;
+#ifdef HAVE_IPV6
+	struct sockaddr_in6 saddr6;
+#endif
+	int domain, one, result;
 
 	while (binding != NULL) {
-		if ((binding->socket = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
+#ifdef HAVE_IPV6
+		domain = (binding->interface.family == AF_INET ? PF_INET : PF_INET6);
+#else
+		domain = PF_INET;
+#endif
+		if ((binding->socket = socket(domain, SOCK_STREAM, 0)) == -1) {
 			perror("socket()");
 			return -1;
 		}
 
-		memset(&saddr, 0, sizeof(struct sockaddr_in));
-		saddr.sin_family = AF_INET;
-		saddr.sin_port = htons(binding->port);
-		saddr.sin_addr.s_addr = binding->interface;
-
 		one = 1;
 		if (setsockopt(binding->socket, SOL_SOCKET, SO_REUSEADDR, (void*)&one, sizeof(int)) == -1) {
 			perror("setsockopt(SOL_SOCKET, SO_REUSEADDR)");
@@ -1283,11 +1290,47 @@ int bind_sockets(t_binding *binding) {
 			perror("setsockopt(IPPROTO_TCP, TCP_NODELAY)");
 		}
 
-		if (bind(binding->socket, (struct sockaddr*)&saddr, sizeof(struct sockaddr)) == -1) {
-			ip_address = (char*)&(binding->interface);
-			printf("Error binding %hhu.%hhu.%hhu.%hhu:%d\n", ip_address[0], ip_address[1], ip_address[2], ip_address[3], binding->port);
+		if (binding->interface.family == AF_INET) {
+			/* IPv4
+			 */
+			memset(&saddr4, 0, sizeof(struct sockaddr_in));
+			//saddr4.sin_len = sizeof(struct sockaddr_in);
+			saddr4.sin_family = AF_INET;
+			memcpy(&(saddr4.sin_addr.s_addr), &(binding->interface.value), IPv4_LEN);
+			saddr4.sin_port = htons(binding->port);
+
+			result = bind(binding->socket, (struct sockaddr*)&saddr4, sizeof(struct sockaddr_in));
+
+			separator = ':';
+#ifdef HAVE_IPV6
+		} else if (binding->interface.family == AF_INET6) {
+			/* IPv6
+			 */
+			memset(&saddr6, 0, sizeof(struct sockaddr_in6));
+			//saddr6.sin6_len = sizeof(struct sockaddr_in6);
+			saddr6.sin6_family = AF_INET6;
+			memcpy(&(saddr6.sin6_addr.s6_addr), &(binding->interface.value), IPv6_LEN);
+			saddr6.sin6_port = htons(binding->port);
+
+			result = bind(binding->socket, (struct sockaddr*)&saddr6, sizeof(struct sockaddr_in6));
+
+			separator = '.';
+#endif
+		} else {
+			fprintf(stderr, "Unknown protocol (family %d).\n", binding->interface.family);
 			return -1;
 		}
+
+		if (result == -1) {
+			/* Handle error
+		 	 */
+			if (inet_ntop(binding->interface.family, &(binding->interface.value), ip_address, MAX_IP_STR_LEN) == NULL) {
+				strcpy(ip_address, "?.?.?.?");
+			}
+			fprintf(stderr, "Error binding %s%c%d\n", ip_address, separator, binding->port);
+			return -1;
+		}
+
 		binding = binding->next;
 	}
 
@@ -1297,13 +1340,16 @@ int bind_sockets(t_binding *binding) {
 /* Accept or deny an incoming connection.
  */
 int accept_connection(t_binding *binding, t_config *config) {
-	socklen_t          size;
-	bool               kick_client;
-	t_session          *session;
-	struct sockaddr_in caddr;
-	pthread_attr_t     child_attr;
-	pthread_t          child_thread;
-	int                total_conns, one;
+	socklen_t           size;
+	bool                kick_client;
+	t_session           *session;
+	struct sockaddr_in  caddr4;
+#ifdef HAVE_IPV6
+	struct sockaddr_in6 caddr6;
+#endif
+	pthread_attr_t      child_attr;
+	pthread_t           child_thread;
+	int                 total_conns, one;
 
 	if ((session = (t_session*)malloc(sizeof(t_session))) == NULL) {
 		return -1;
@@ -1312,19 +1358,48 @@ int accept_connection(t_binding *binding, t_config *config) {
 	session->binding = binding;
 	init_session(session);
 
-	size = sizeof(struct sockaddr_in);
-	memset((void*)&caddr, 0, (size_t)size);
-	if ((session->client_socket = accept(binding->socket, (struct sockaddr*)&caddr, &size)) == -1) {
-		free(session);
-		if (errno != EINTR) {
-			return -1;
+	if (binding->interface.family == AF_INET) {
+		/* IPv4
+		 */
+		size = sizeof(struct sockaddr_in);
+		memset((void*)&caddr4, 0, (size_t)size);
+		if ((session->client_socket = accept(binding->socket, (struct sockaddr*)&caddr4, &size)) == -1) {
+			free(session);
+			if (errno != EINTR) {
+				return -1;
+			}
+			return 0;
 		}
-		return 0;
+
+		session->ip_address.family = AF_INET;
+		session->ip_address.size   = IPv4_LEN;
+		memcpy(&(session->ip_address.value), (char*)&caddr4.sin_addr.s_addr, session->ip_address.size);
+#ifdef HAVE_IPV6
+	} else if (binding->interface.family == AF_INET6) {
+		/* IPv6
+		 */
+		size = sizeof(struct sockaddr_in6);
+		memset((void*)&caddr6, 0, (size_t)size);
+		if ((session->client_socket = accept(binding->socket, (struct sockaddr*)&caddr6, &size)) == -1) {
+			free(session);
+			if (errno != EINTR) {
+				return -1;
+			}
+			return 0;
+		}
+
+		session->ip_address.family = AF_INET6;
+		session->ip_address.size   = IPv6_LEN;
+		memcpy(&(session->ip_address.value), (char*)&caddr6.sin6_addr.s6_addr, session->ip_address.size);
+#endif
+	} else {
+		log_system(session, "Incoming connection via unknown protocol");
+		return -1;
 	}
 
 	kick_client = true;
-	memcpy((char*)&(session->ip_address), (char*)&caddr.sin_addr.s_addr, 4);
-	if ((total_conns = connection_allowed(session->ip_address, config->connections_per_ip, config->total_connections)) >= 0) {
+
+	if ((total_conns = connection_allowed(&(session->ip_address), config->connections_per_ip, config->total_connections)) >= 0) {
 		if (total_conns < (config->total_connections >> 2)) {
 			one = 1;
 			if (setsockopt(session->client_socket, IPPROTO_TCP, TCP_NODELAY, (void*)&one, sizeof(int)) == -1) {
@@ -1351,8 +1426,8 @@ int accept_connection(t_binding *binding, t_config *config) {
 		}
 	} else switch (total_conns) {
 		case ca_TOOMUCH_PERIP:
-			if ((config->ban_on_max_per_ip > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
-				ban_ip(session->ip_address, config->ban_on_max_per_ip, config->kick_on_ban);
+			if ((config->ban_on_max_per_ip > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+				ban_ip(&(session->ip_address), config->ban_on_max_per_ip, config->kick_on_ban);
 			}
 			log_system(session, "Maximum number of connections for IP address reached");
 			break;
@@ -1360,8 +1435,8 @@ int accept_connection(t_binding *binding, t_config *config) {
 			log_system(session, "Maximum number of total connections reached");
 			break;
 		case ca_BANNED:
-			if (config->reban_during_ban && ip_allowed(session->ip_address, session->config->banlist_mask)) {
-				reban_ip(session->ip_address);
+			if (config->reban_during_ban && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+				reban_ip(&(session->ip_address));
 			}
 #ifdef HAVE_COMMAND
 			increment_counter(COUNTER_DENY);
@@ -1437,7 +1512,6 @@ int run_server(t_settings *settings) {
 	/* Bind Serverports
 	 */
 	if (bind_sockets(config->binding) == -1) {
-		perror("bind http");
 		return EXIT_FAILURE;
 	}
 
@@ -1464,7 +1538,6 @@ int run_server(t_settings *settings) {
 	/* Bind CommandChannels
 	 */
 	if (bind_sockets(config->command_port) == -1) {
-		perror("bind command");
 		return EXIT_FAILURE;
 	}
 #endif
@@ -1673,7 +1746,7 @@ int run_server(t_settings *settings) {
 				}
 #endif
 
-				/* HTTP(S) port */
+				/* HTTP(S) ports */
 				binding = config->binding;
 				while (binding != NULL) {
 					if (FD_ISSET(binding->socket, &read_fds)) {
@@ -1688,7 +1761,7 @@ int run_server(t_settings *settings) {
 				}
 
 #ifdef HAVE_COMMAND
-				/* Command port */
+				/* Command ports */
 				binding = config->command_port;
 				while (binding != NULL) {
 					if (FD_ISSET(binding->socket, &read_fds)) {
@@ -1729,6 +1802,7 @@ int run_server(t_settings *settings) {
 #endif
 
 	log_string(config->system_logfile, "Hiawatha v"VERSION" stopped");
+	log_close(config->first_host, true);
 
 	return EXIT_SUCCESS;
 }
