@@ -278,12 +278,12 @@ static bool digest_http_authentication(t_session *session, char *auth_str) {
 
 /* Check if the file is protected by an .hiawatha file with passwordfile setting.
  */
-bool http_authentication_oke(t_session *session) {
+bool http_authentication_oke(t_session *session, bool on_pwdfile_missing) {
 	char *auth_str;
 	bool result = false;
 
 	if (session->host->passwordfile == NULL) {
-		return true;
+		return on_pwdfile_missing;
 	} else if ((auth_str = get_headerfield("Authorization:", session->headerfields)) != NULL) {
 		auth_str = strdup(auth_str);
 		if ((strncmp(auth_str, "Basic ", 6) == 0) && (session->host->auth_method == basic)) {
@@ -292,9 +292,7 @@ bool http_authentication_oke(t_session *session) {
 			result = digest_http_authentication(session, auth_str + 7);
 		}
 		free(auth_str);
-
-		return result;
 	}
 
-	return false;
+	return result;
 }
