@@ -6,7 +6,7 @@
 #include <sys/types.h>
 #include "userconfig.h"
 
-enum t_fsbool { error = -3, not_found, no_access, no, yes };
+typedef enum { error = -3, not_found, no_access, no, yes } t_fsbool;
 
 typedef struct type_filelist {
 	char   *name;
@@ -19,14 +19,17 @@ typedef struct type_filelist {
 
 off_t filesize(char *filename);
 char *make_path(char *dir, char *file);
-enum t_fsbool contains_not_allowed_symlink(char *filename, char *webroot);
-enum t_fsbool is_directory(char *file);
-enum t_fsbool can_execute(char *file, uid_t uid, gid_t gid, t_groups *groups);
+t_fsbool contains_not_allowed_symlink(char *filename, char *webroot);
+t_fsbool is_directory(char *file);
+t_fsbool can_execute(char *file, uid_t uid, gid_t gid, t_groups *groups);
 void touch_logfile(char *dir, char *file, mode_t mode, uid_t uid, gid_t gid);
 int  if_modified_since(int handle, char *datestr);
 FILE *fopen_neighbour(char *filename, char *mode, char *neighbour);
 t_filelist *read_filelist(char *directory);
 t_filelist *sort_filelist(t_filelist *filelist);
 void remove_filelist(t_filelist *filelist);
+#ifdef CYGWIN
+char *cygwin_to_windows(char *path);
+#endif
 
 #endif
