@@ -2,9 +2,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/socket.h>
 #include "global.h"
 #include "libstr.h"
 #include "liblist.h"
+#include "libip.h"
 
 void sfree(void *ptr) {
 	if (ptr != NULL) {
@@ -12,50 +14,6 @@ void sfree(void *ptr) {
 	}
 }
 
-/* Convert an IP address to an unsigned long.
- */
-int parse_ip(char *line, t_ipv4 *l_ip) {
-	char *ip, *c, org;
-	int  i, byte;
-
-	if (line == NULL) {
-		return -1;
-	}
-	if (*line == '\0') {
-		return -1;
-	}
-
-	ip = (char*)l_ip;
-	i = 0;
-	c = line - 1;
-	do {
-		c++;
-		switch (*c) {
-			case '\0':
-				if (i < 3) {
-					return -1;
-				}
-			case '.':
-				if (i > 3) {
-					return -1;
-				}
-				org = *c;
-				*c = '\0';
-				byte = str2int(line);
-				*c = org;
-				if ((byte < 0) || (byte > 255)) {
-					return -1;
-				}
-				*(ip + i) = (char)byte;
-				line = c + 1;
-				i++;
-				break;
-		}
-	} while (*c != '\0');
-
-	return 0;
-}
-
 /*---< charlist >-------------------------------------------------------------*/
 
 void init_charlist(t_charlist *list) {
@@ -159,10 +117,9 @@ t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list)
 
 	while (line != NULL) {
 		split_string(line, &ip, &line, ',');
-		if (split_string(ip, &rule, &ip, ':') == 0) {
+		if (split_string(ip, &rule, &ip, ' ') == 0) {
 			if (list == NULL) {
 				if ((list = new = (t_accesslist*)malloc(sizeof(t_accesslist))) == NULL) {
-					error = true;
 					break;
 				}
 			} else {
@@ -181,30 +138,68 @@ t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list)
 			} else if (pwd_allowed && (strcmp(rule, "pwd") == 0)) {
 				new->access = pwd;
 			} else {
-				list = remove_accesslist(list);
+				error = true;
 				break;
 			}
 			if (strcmp(ip, "all") == 0) {
-				new->ip = 0;
-				new->netmask = 32;
+				default_ipv4(&(new->ip));
+				new->netmask = 0;
+				new->all_ip = true;
 			} else {
+				new->all_ip = false;
 				if (split_string(ip, &ip, &mask, '/') == 0) {
 					if ((new->netmask = str2int(mask)) == -1) {
 						error = true;
 						break;
-					} else if (new->netmask > 32) {
-						error = true;
-						break;
-					} else {
-						new->netmask = 32 - new->netmask;
 					}
 				} else {
-					new->netmask = 0;
+					new->netmask = -1;
 				}
 				if (parse_ip(ip, &(new->ip)) == -1) {
 					error = true;
 					break;
 				}
+
+				if (new->netmask == -1) {
+					/* Set netmask
+					 */
+					if (new->ip.family == AF_INET) {
+						new->netmask = 8 * IPv4_LEN;
+#ifdef HAVE_IPV6
+					} else if (new->ip.family == AF_INET6) {
+						new->netmask = 8 * IPv6_LEN;
+#endif
+					} else {
+						error = true;
+						break;
+					}
+				} else {
+					/* Check netmask
+					 */
+					if (new->ip.family == AF_INET) {
+						if (new->netmask > 8 * IPv4_LEN) {
+							error = true;
+							break;
+						}
+#ifdef HAVE_IPV6
+					} else if (new->ip.family == AF_INET6) {
+						if (new->netmask > 8 * IPv6_LEN) {
+							error = true;
+							break;
+						}
+#endif
+					} else {
+						error = true;
+						break;
+					}
+				}
+
+				/* Apply subnetmask
+				 */
+				if (apply_netmask(&(new->ip), new->netmask) == -1) {
+					error = true;
+					break;
+				}
 			}
 		} else {
 			error = true;
@@ -222,13 +217,13 @@ t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list)
 /* Remove an accesslist.
  */
 t_accesslist *remove_accesslist(t_accesslist *list) {
-	t_accesslist *ip;
+	t_accesslist *item;
 
 	while (list != NULL) {
-		ip = list;
+		item = list;
 		list = list->next;
 
-		free(ip);
+		free(item);
 	}
 	
 	return NULL;
@@ -236,12 +231,11 @@ t_accesslist *remove_accesslist(t_accesslist *list) {
 
 /* Return the access status of an IP address.
  */
-#define masked(ip, mask) ((ip << mask) >> mask)
-t_access ip_allowed(t_ipv4 ip, t_accesslist *list) {
+t_access ip_allowed(t_ip_addr *ip, t_accesslist *list) {
 	while (list != NULL) {
-		if (list->netmask == 32) {
+		if (list->all_ip) {
 			return list->access;
-		} else if (masked(ip, list->netmask) == masked(list->ip, list->netmask)) {
+		} else if (ip_in_subnet(ip, &(list->ip), list->netmask)) {
 			return list->access;
 		}
 		list = list->next;
