@@ -3,6 +3,7 @@
 
 #include <stdbool.h>
 #include "global.h"
+#include "libip.h"
 
 typedef enum { deny, allow, pwd, unknown } t_access;
 typedef enum { tc_data, tc_charlist, tc_accesslist, tc_keyvalue } t_tempdata_type;
@@ -13,8 +14,9 @@ typedef struct type_charlist {
 } t_charlist;
 
 typedef struct type_accesslist {
-	t_ipv4 ip;
+	t_ip_addr ip;
 	int netmask;
+	bool all_ip;
 	t_access access;
 
 	struct type_accesslist *next;
@@ -49,10 +51,9 @@ void copy_charlist(t_charlist *dest, t_charlist *src);
 bool in_charlist(char *item, t_charlist *list);
 void remove_charlist(t_charlist *list);
 
-int  parse_ip(char *line, t_ipv4 *l_ip);
 t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list);
 t_accesslist *remove_accesslist(t_accesslist *list);
-t_access ip_allowed(t_ipv4 ip, t_accesslist *list);
+t_access ip_allowed(t_ip_addr *ip, t_accesslist *list);
 
 int  parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *delimiter);
 t_keyvalue *remove_keyvaluelist(t_keyvalue *list);
