@@ -10,11 +10,12 @@
 #include <string.h>
 #include <time.h>
 #include <pthread.h>
+#include "global.h"
 #include "log.h"
 
-#define BUFFER_SIZE     2000
+#define BUFFER_SIZE        2 * KILOBYTE
 #define TIMESTAMP_SIZE    40
-#define IP_ADDRESS_SIZE   20
+#define IP_ADDRESS_SIZE   MAX_IP_STR_LEN + 2
 
 #define MAX_LOG_DELAY_TIMER 10
 #define LOGFILE_OPEN_TIME   30
@@ -41,12 +42,12 @@ static void print_timestamp(char *str) {
 
 /* Write an IP address to a logfile.
  */
-static void print_ip_address(char *str, t_ipv4 ip_address) {
-	unsigned char *ip;
-
-	ip = (unsigned char*)&ip_address;
-	str[IP_ADDRESS_SIZE - 1] = '\0';
-	snprintf(str, IP_ADDRESS_SIZE - 1, "%hhu.%hhu.%hhu.%hhu|", ip[0], ip[1], ip[2], ip[3]);
+static void print_ip_address(char *str, t_ip_addr *ip) {
+	if (inet_ntop(ip->family, &(ip->value), str, IP_ADDRESS_SIZE) == NULL) {
+		strcpy(str, "?.?.?.?|");
+	} else {
+		strcat(str, "|");
+	}
 }
 
 static char *secure_string(char *str) {
@@ -102,7 +103,7 @@ void log_system(t_session *session, char *mesg) {
 	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
 	if ((fp = fopen(session->config->system_logfile, "a")) != NULL) {
-		print_ip_address(str, session->ip_address);
+		print_ip_address(str, &(session->ip_address));
 		print_timestamp(str + strlen(str));
 		fprintf(fp, "%s%s\n", str, mesg);
 		fclose(fp);
@@ -117,7 +118,7 @@ void log_error(t_session *session, char *mesg) {
 
 	if ((fp = fopen(session->host->error_logfile, "a")) != NULL) {
 		print_timestamp(str);
-		print_ip_address(str + strlen(str), session->ip_address);
+		print_ip_address(str + strlen(str), &(session->ip_address));
 		fprintf(fp, "%s%s\n", str, mesg);
 		fclose(fp);
 	}
@@ -133,7 +134,7 @@ void log_request(t_session *session, int code) {
 	time_t t;
 	struct tm *s;
 
-	if (ip_allowed(session->ip_address, session->config->logfile_mask) != deny) {
+	if (ip_allowed(&(session->ip_address), session->config->logfile_mask) != deny) {
 		str[BUFFER_SIZE] = '\0';
 
 #ifdef HAVE_REWRITE
@@ -161,7 +162,7 @@ void log_request(t_session *session, int code) {
 			/* Hiawatha log format
 			 */
 
-			print_ip_address(str, session->ip_address);
+			print_ip_address(str, &(session->ip_address));
 			offset = strlen(str);
 			print_timestamp(str + offset);
 			offset += strlen(str + offset);
@@ -198,7 +199,7 @@ void log_request(t_session *session, int code) {
 		} else {
 			/* Common Log Format
 			 */
-			print_ip_address(ip_address, session->ip_address);
+			print_ip_address(ip_address, &(session->ip_address));
 			if ((len = strlen(ip_address)) > 0) {
 				ip_address[len - 1] = '\0';
 			}
@@ -250,6 +251,7 @@ void log_request(t_session *session, int code) {
 		}
 		if (*(session->host->access_fp) != NULL) {
 			fprintf(*(session->host->access_fp), "%s\n", str);
+			fflush(*(session->host->access_fp));
 		}
 		pthread_mutex_unlock(&accesslog_mutex);
 	}
@@ -263,7 +265,7 @@ void log_garbage(t_session *session) {
 
 	if ((session->config->garbage_logfile != NULL) && (session->request != NULL)) {
 		if ((fp = fopen(session->config->garbage_logfile, "a")) != NULL) {
-			print_ip_address(str, session->ip_address);
+			print_ip_address(str, &(session->ip_address));
 			print_timestamp(str + strlen(str));
 			fprintf(fp, "%s%s\n", str, session->request);
 			fclose(fp);
@@ -273,7 +275,7 @@ void log_garbage(t_session *session) {
 
 /* Log an unbanning.
  */
-void log_unban(char *logfile, t_ipv4 ip_address, unsigned long connect_attempts) {
+void log_unban(char *logfile, t_ip_addr *ip_address, unsigned long connect_attempts) {
 	FILE *fp;
 	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
@@ -309,7 +311,7 @@ void log_file_error(t_session *session, char *file, char *mesg) {
 	if (len > 0) {
 		if ((fp = fopen(session->host->error_logfile, "a")) != NULL) {
 			print_timestamp(str);
-			print_ip_address(str + strlen(str), session->ip_address);
+			print_ip_address(str + strlen(str), &(session->ip_address));
 			fprintf(fp, "%s%s|%s\n", str, file, secure_string(mesg));
 			fclose(fp);
 		}
@@ -329,7 +331,7 @@ void log_close(t_host *host, bool force_close) {
 		while (host != NULL) {
 			if ((now >= host->access_time + LOGFILE_OPEN_TIME) || force_close) {
 				if (*(host->access_fp) != NULL) {
-					fflush(*(host->access_fp));
+					//fflush(*(host->access_fp));
 					fclose(*(host->access_fp));
 					*(host->access_fp) = NULL;
 				}
