@@ -14,7 +14,7 @@ static char *default_mimetype = "text/plain";
  */
 int read_mimetypes(char *configfile, t_mimetype **mime) {
 	FILE *fp;
-	char *line, *data, *next, *ext;
+	char line[257], *data, *next, *ext;
 	bool quit;
 	t_mimename *w_mime = NULL;
 	t_extension *w_ext = NULL;
@@ -28,15 +28,12 @@ int read_mimetypes(char *configfile, t_mimetype **mime) {
 		fclose(fp);
 		return -1;
 	}
-	if ((line = (char*)malloc(257)) == NULL) {
-		fclose(fp);
-		free(*mime);
-		return -1;
-	}
-	*(line + 256) = '\0';
+	line[256] = '\0';
 
 	while (fgets(line, 256, fp) != NULL) {
-		next = data = strdup(line);
+		if ((next = data = strdup(line)) == NULL) {
+			return -1;
+		}
 		while (*next != '\0') {
 			if (*next == '\t') {
 				*next = ' ';
@@ -94,8 +91,6 @@ int read_mimetypes(char *configfile, t_mimetype **mime) {
 			}
 		}
 	}
-
-	free(line);
 	fclose(fp);
 
 	return 0;
