@@ -11,8 +11,8 @@
 #include "liblist.h"
 #include "userconfig.h"
 
-#define MAX_CONFIG_LINE 1024
-#define DELIMITER ','
+#define MAX_CONFIG_LINE KILOBYTE
+#define DELIMITER ';'
 
 /* Type definitions
  */
@@ -131,23 +131,9 @@ int parse_line(t_config *config, char *line) {
 				return -1;
 			}
 
-			/* Syntax-bug workaround.
-			 */
-			item = value;
-			if ((value = strrchr(item, ',')) != NULL) {
-				if (str2int(value + 1) == -1) {
-					*value = '\0';
-					value = remove_spaces(value + 1);
-				} else {
-					value = NULL;
-				}
-			} else {
-				value = NULL;
-			}
-			item = remove_spaces(item);
-
 			/* UID, GID's
 			 */
+			split_string(value, &item, &value, DELIMITER);
 			split_string(item, &item, &group, ':');
 			if (parse_userid(item, &(new->uid)) != 1) {
 				return -1;
