@@ -397,7 +397,7 @@ int send_code(t_session *session, int code) {
 		emesg = unknown_code;
 	}
 	len[9] = '\0';
-	snprintf(len, 9, "%d", 2 * (int)strlen(emesg) + 291);
+	snprintf(len, 9, "%d", 2 * (int)strlen(emesg) + 380);
 
 	if (send_buffer(session, "Content-Length: ", 16) == -1) {
 		return -1;
@@ -411,7 +411,9 @@ int send_code(t_session *session, int code) {
 		return -1;
 	} else if (session->head_request) {
 		return 0;
-	} else if (send_buffer(session, "<html><head><title>", 19) == -1) {
+	} else if (send_buffer(session, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n", 91) == -1) {
+		return -1;
+	} else if (send_buffer(session, "<html>\n<head>\n<title>", 21) == -1) {
 		return -1;
 	} else if (send_buffer(session, ecode, 3) == -1) {
 		return -1;
@@ -419,13 +421,12 @@ int send_code(t_session *session, int code) {
 		return -1;
 	} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
 		return -1;
-	} else if (send_buffer(session,	"</title><style type=\"text/css\">\n" /* 32 */
-									"<!--\n" /* 5 */
-									"BODY { color:#ffffff ; background-color:#00000a }\n" /* 50 */
-									"DIV { font-family:sans-serif ; font-size:30px ; letter-spacing:20px ; " /* 70 */
-									"text-align:center ; position:relative ; top:250px }\n" /* 52 */
-									"--></style></head>\n" /* 19 */
-									"<body><div>", 239) == -1) { /* 11 */
+	} else if (send_buffer(session,	"</title>\n<style type=\"text/css\">" /* 32 */
+	                                "BODY { color:#ffffff ; background-color:#00000a }\n" /* 50 */
+	                                "DIV { font-family:sans-serif ; font-size:30px ; letter-spacing:20px ; " /* 70 */
+	                                "text-align:center ; position:relative ; top:250px }\n" /* 52 */
+	                                "</style>\n</head>\n" /* 17 */
+	                                "<body>\n<div>", 233) == -1) { /* 12 */
 		return -1;
 	} else if (send_buffer(session, ecode, 3) == -1) {
 		return -1;
@@ -433,7 +434,7 @@ int send_code(t_session *session, int code) {
 		return -1;
 	} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
 		return -1;
-	} else if (send_buffer(session, "</div></body></html>\n", 21) == -1) {
+	} else if (send_buffer(session, "</div>\n</body>\n</html>\n", 23) == -1) {
 		return -1;
 	}
 
