@@ -2,9 +2,10 @@
 #define _SEND_H
 
 #include <stdbool.h>
+#include "global.h"
 #include "session.h"
 
-#define FCGI_BUFFER_SIZE 65535
+#define FCGI_BUFFER_SIZE 64 * KILOBYTE
 
 typedef struct {
 	int           sock;
