@@ -160,7 +160,7 @@ static t_binding *new_binding(void) {
 
 	if ((binding = (t_binding*)malloc(sizeof(t_binding))) != NULL) {
 		binding->port                 = -1;
-		binding->interface            = 0;
+		default_ipv4(&(binding->interface));
 #ifdef HAVE_SSL
 		binding->use_ssl              = false;
 		binding->server_key           = NULL;
@@ -173,8 +173,8 @@ static t_binding *new_binding(void) {
 		binding->enable_trace         = false;
 		binding->enable_alter         = false;
 		binding->max_keepalive        = 50;
-		binding->max_request_size     = 65536;
-		binding->max_upload_size      = 1048576;
+		binding->max_request_size     = 64 * KILOBYTE;
+		binding->max_upload_size      = MEGABYTE;
 		binding->time_for_1st_request = 5;
 		binding->time_for_request     = 30;
 
@@ -273,8 +273,8 @@ t_config *default_config(void) {
 #endif
 
 #ifdef HAVE_CACHE
-		config->cache_size         = 10485760;  /*  10MB */
-		config->cache_max_filesize = 262144;    /* 256Kb */
+		config->cache_size         = 10 * MEGABYTE;
+		config->cache_max_filesize = 256 * KILOBYTE;
 		config->cache_min_filesize = 0;
 #endif
 
@@ -678,7 +678,7 @@ static bool system_setting(char *key, char *value, t_config *config) {
 #ifdef HAVE_CACHE
 	} else if (strcmp(key, "cachemaxfilesize") == 0) {
 		if ((config->cache_max_filesize = str2int(value)) != -1) {
-			config->cache_max_filesize <<= 10;
+			config->cache_max_filesize <<= 10 /* convert to kB */;
 			return true;
 		}
 	} else if (strcmp(key, "cacheminfilesize") == 0) {
@@ -688,7 +688,7 @@ static bool system_setting(char *key, char *value, t_config *config) {
 	} else if (strcmp(key, "cachesize") == 0) {
 		if ((config->cache_size = str2int(value)) != -1) {
 			if (config->cache_size <= MAX_CACHE_SIZE) {
-				config->cache_size <<= 20;
+				config->cache_size <<= 20 /* convert to MB */;
 				return true;
 			}
 		}
@@ -697,9 +697,11 @@ static bool system_setting(char *key, char *value, t_config *config) {
 	} else if (strcmp(key, "commandchannel") == 0) {
 		if (split_string(value, &port, &password, ',') == 0) {
 			binding = new_binding();
+			set_to_localhost(&(binding->interface));
+
 			binding->next = config->command_port;
 			config->command_port = binding;
-			config->command_port->interface = htonl(0x7F000001);
+
 			if ((config->command_port->port = str2int(port)) > 0) {
 				if ((config->command_port->binding_id = strdup(password)) != NULL) {
 					return true;
@@ -875,7 +877,7 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				throt->next = NULL;
 
 				if ((throt->filetype = strlower(strdup(rest))) != NULL) {
-					throt->upload_speed = speed << 10; /* convert to kilobytes */
+					throt->upload_speed = speed << 10; /* convert to kB/s */
 					return true;
 				}
 			}
@@ -1263,7 +1265,7 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 	} else if (strcmp(key, "uploadspeed") == 0) {
 		if (split_string(value, &value, &maxclients, ',') == 0) {
 			if ((directory->upload_speed = str2int(value)) > 0) {
-				directory->upload_speed <<= 10;
+				directory->upload_speed <<= 10 /* convert to kB/s */;
 				if ((directory->max_clients = str2int(maxclients)) > 0) {
 					return true;
 				}
@@ -1291,7 +1293,7 @@ static bool binding_setting(char *key, char *value, t_binding *binding) {
 			return true;
 		}
 	} else if (strcmp(key, "interface") == 0) {
-		if (parse_ip(value, &(binding->interface)) == 0) {
+		if (parse_ip(value, &(binding->interface)) != -1) {
 			return true;
 		}
 	} else if (strcmp(key, "maxkeepalive") == 0) {
@@ -1300,13 +1302,13 @@ static bool binding_setting(char *key, char *value, t_binding *binding) {
 		}
 	} else if (strcmp(key, "maxrequestsize") == 0) {
 		if ((binding->max_request_size = str2int(value)) > 0) {
-			binding->max_request_size <<= 10;
+			binding->max_request_size <<= 10 /* convert to kB */;
 			return true;
 		}
 	} else if (strcmp(key, "maxuploadsize") == 0) {
 		if ((binding->max_upload_size = str2int(value)) > 0) {
 			if (binding->max_upload_size <= MAX_UPLOAD_SIZE) {
-				binding->max_upload_size <<= 20;
+				binding->max_upload_size <<= 20 /* convert to MB */;
 				return true;
 			}
 		}
@@ -1378,9 +1380,9 @@ static bool fcgi_server_setting(char *key, char *value, t_fcgi_server *fcgi_serv
 			connect_to->available = true;
 			fcgi_server->connect_to = connect_to;
 
-			if (split_string(value, &value, &port, ':') != 0) {
+			if (split_string(value, &value, &port, ' ') != 0) {
 				return false;
-			} else if ((connect_to->host = strdup(value)) == NULL) {
+			} else if (parse_ip(value, &(connect_to->host)) == -1) {
 				return false;
 			} else if ((connect_to->port = str2int(port)) <= 0) {
 				return false;
@@ -1572,7 +1574,7 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 						}
 						break;
 					case fcgi_server:
-						if ((config->fcgi_server->fcgi_id != NULL) && (config->fcgi_server->connect_to != NULL)) {
+						if ((config->fcgi_server->fcgi_id != NULL) && (config->fcgi_server->connect_to->port != 0)) {
 							current_fcgi_server = NULL;
 						} else {
 							fprintf(stderr, "A FastCGIid and/or ConnectTo is missing in a FastCGIserver section in %s.\n", configfile);
