@@ -1,10 +1,10 @@
 #ifndef _SERVERCONFIG_H
 #define _SERVERCONFIG_H
 
-#include "config.h"
 #include <stdbool.h>
 #include <pthread.h>
 #include "global.h"
+#include "libip.h"
 #include "mimetype.h"
 #ifdef HAVE_REWRITE
 #include "rewrite.h"
@@ -31,14 +31,14 @@ typedef struct type_cgi_handler {
 
 typedef struct type_connect_to {
 	int           port;
-	char          *host;
+	t_ip_addr     host;
 	bool          available;
 
 	struct type_connect_to *next;
 } t_connect_to;
 
 typedef struct type_cgi_session {
-	t_ipv4        client_ip;
+	t_ip_addr     client_ip;
 	t_connect_to  *connect_to;
 	int           timer;
 
@@ -68,7 +68,7 @@ typedef struct type_throttle {
 
 typedef struct type_binding {
 	int           port;
-	t_ipv4        interface;
+	t_ip_addr     interface;
 	char          *binding_id;
 #ifdef HAVE_SSL
 	bool          use_ssl;
