@@ -11,6 +11,7 @@
 #include <string.h>
 #include <pthread.h>
 #include <pwd.h>
+#include "alternative.h"
 #include "libstr.h"
 #include "liblist.h"
 #include "session.h"
@@ -451,13 +452,12 @@ void close_socket(t_session *session) {
 			ssl_close(session->ssl_data);
 		}
 #endif
+		fsync(session->client_socket);
 		close(session->client_socket);
 		session->socket_open = false;
 	}
 }
 
-// --< debug >-----------------------------------------------------------------
-
 #ifdef DEBUG
 void set_client_status(t_session *session, char *status) {
 	sfree(session->status);
