@@ -1,16 +1,16 @@
 #ifndef _SESSION_H
 #define _SESSION_H
 
-#include "config.h"
 #include <time.h>
 #ifdef HAVE_SSL
 #include "libssl.h"
 #endif
+#include "libip.h"
 #include "liblist.h"
 #include "serverconfig.h"
 
-#define MAX_CHILD_ID 30000
-#define OUTPUT_BUFFER_SIZE 2000
+#define MAX_CHILD_ID        30000
+#define OUTPUT_BUFFER_SIZE      2 * KILOBYTE
 
 typedef enum { none, program, script, fastcgi} t_cgi_type;
 #ifdef HAVE_SSL
@@ -51,7 +51,7 @@ typedef struct type_session {
 	bool            encode_gzip;
 	bool            alias_used;
 	t_headerfield   *headerfields;
-	t_ipv4          ip_address;
+	t_ip_addr       ip_address;
 	bool            ip_resolved;
 	char            *mimetype;
 	char            *hostname;
