@@ -39,14 +39,14 @@
 #include "command.h"
 #endif
 
-#define MAX_VOLATILE_SIZE    1048576
-#define FILE_BUFFER_SIZE       32768
-#define MAX_CGI_HEADER          8192
-#define MAX_TRACE_HEADER        1024
-#define VALUE_SIZE                64
-#define WAIT_FOR_LOCK              3
-#define FILESIZE_BUFFER_SIZE      30
-#define CGI_BUFFER_SIZE        32768
+#define MAX_VOLATILE_SIZE      1 * MEGABYTE
+#define FILE_BUFFER_SIZE      32 * KILOBYTE
+#define MAX_CGI_HEADER         8 * KILOBYTE
+#define MAX_TRACE_HEADER       1 * KILOBYTE
+#define VALUE_SIZE            64
+#define WAIT_FOR_LOCK          3
+#define FILESIZE_BUFFER_SIZE  30
+#define CGI_BUFFER_SIZE       32 * KILOBYTE
 
 #define rs_QUIT       -1
 #define rs_DISCONNECT -2
@@ -612,7 +612,7 @@ int execute_cgi(t_session *session) {
 
 	if (session->cgi_type == fastcgi) {
 		cgi_info.read_header = true;
-		if ((connect_to = select_connect_to(session->fcgi_server, session->ip_address)) == NULL) {
+		if ((connect_to = select_connect_to(session->fcgi_server, &(session->ip_address))) == NULL) {
 			return 503;
 		} else if ((cgi_info.from_cgi = connect_to_fcgi_server(connect_to)) == -1) {
 			connect_to->available = false;
@@ -1008,7 +1008,9 @@ int show_index(t_session *session) {
 			break;
 		} else if (send_buffer(session, NULL, 0) == -1) {
 			break;
-		} else if (send_chunk(session, "<html><head><title>Index of ", 28) == -1) {
+		} else if (send_chunk(session, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n", 91) == -1) {
+			break;
+		} else if (send_chunk(session, "<html>\n<head>\n<title>Index of ", 30) == -1) {
 			break;
 		} else if (send_chunk(session, session->uri, session->uri_len) == -1) {
 			break;
@@ -1030,7 +1032,7 @@ int show_index(t_session *session) {
 			}
 		}
 
-		if (send_chunk(session,"<body><div class=\"path\">", 24) == -1) { 
+		if (send_chunk(session,"</head>\n<body>\n<div class=\"path\">", 33) == -1) { 
 			break;
 		} else if (send_chunk(session, *(session->host->hostname.item), strlen(*(session->host->hostname.item))) == -1) {
 			break;
@@ -1038,7 +1040,7 @@ int show_index(t_session *session) {
 			break;
 		} else if (send_chunk(session, session->uri, session->uri_len) == -1) {
 			break;
-		} else if (send_chunk(session, "</div>\n<table class=\"list\">\n<tr><th class=\"left\"></th><th class=\"filename\">Filename</th><th class=\"date\">Date</th><th class=\"size\">Size</th><th class=\"right\"></th></tr>\n", 169) == -1) {
+		} else if (send_chunk(session, "</div>\n<table class=\"list\">\n<tr class=\"header\"><th class=\"left\"></th><th class=\"filename\">Filename</th><th class=\"date\">Date</th><th class=\"size\">Size</th><th class=\"right\"></th></tr>\n", 184) == -1) {
 			break;
 		} 
 
@@ -1131,7 +1133,7 @@ int show_index(t_session *session) {
 		len = filesize2str(size, total_fsize);
 		if (send_chunk(session, size, len) == -1) {
 			break;
-		} else if (send_chunk(session, "</td><td></td></tr>\n</table></body></html>\n", 43) == -1) {
+		} else if (send_chunk(session, "</td><td></td></tr>\n</table>\n</body>\n</html>\n", 45) == -1) {
 			break;
 		} else  if (session->keep_alive) {
 			if (send_chunk(session, NULL, 0) == -1) {
@@ -1285,16 +1287,16 @@ int handle_trace_request(t_session *session) {
 
 static t_access allow_alter(t_session *session) {
 	char *x_forwarded_for;
-	t_ipv4 forwarded_ip;
+	t_ip_addr forwarded_ip;
 	t_access access;
 
-	if ((access = ip_allowed(session->ip_address, session->host->alter_list)) != allow) {
+	if ((access = ip_allowed(&(session->ip_address), session->host->alter_list)) != allow) {
 		return access;
 	} else if ((x_forwarded_for = get_headerfield("X-Forwarded-For:", session->headerfields)) == NULL) {
 		return allow;
 	} else if (parse_ip(x_forwarded_for, &forwarded_ip) == -1) {
 		return allow;
-	} else if (ip_allowed(forwarded_ip, session->host->alter_list) == deny) {
+	} else if (ip_allowed(&forwarded_ip, session->host->alter_list) == deny) {
 		return deny;
 	}
 
@@ -1307,11 +1309,14 @@ int handle_put_request(t_session *session) {
 	ssize_t bytes_read;
 	char *range, *value, *rest, *buffer;
 	bool range_found;
+	struct flock file_lock;
 
 	if (session->uploaded_file == NULL) {
 		return 500;
 	}
 
+	/* Access check
+	 */
 	switch (allow_alter(session)) {
 		case deny:
 		case unknown:
@@ -1320,7 +1325,7 @@ int handle_put_request(t_session *session) {
 		case allow:
 			break;
 		case pwd:
-			if (http_authentication_oke(session) == false) {
+			if (http_authentication_oke(session, false) == false) {
 				return 401;
 			}
 			if (group_oke(session, session->remote_user, &(session->host->alter_group)) == false) {
@@ -1336,6 +1341,8 @@ int handle_put_request(t_session *session) {
 	range = get_headerfield("Content-Range:", session->headerfields);
 	range_found = (range != NULL);
 
+	/* Open file for writing
+	 */
 	if ((handle_write = open(session->file_on_disk, O_WRONLY)) == -1) {
 		/* New file */
 		if (range_found) {
@@ -1355,9 +1362,17 @@ int handle_put_request(t_session *session) {
 		}
 		result = 204;
 	}
-	
+
+	/* Lock file for writing
+	 */
+	file_lock.l_type = F_WRLCK;
+	file_lock.l_whence = SEEK_SET;
+	file_lock.l_start = 0;
+	file_lock.l_len = 0;
+	file_lock.l_pid = 0;
 	lock_timeout = WAIT_FOR_LOCK;
-	while (fcntl(handle_write, F_SETLK, F_WRLCK) == -1) {
+
+	while (fcntl(handle_write, F_SETLK, &file_lock) == -1) {
 		if (errno == EINTR) {
 			continue;
 		} else if ((lock_timeout > 0) && ((errno == EACCES) || (errno == EAGAIN))) {
@@ -1372,10 +1387,15 @@ int handle_put_request(t_session *session) {
 			return 500;
 		}
 	}
+
+	/* Change file mode
+	 */
 	if (fchmod(handle_write, session->host->alter_fmode) == -1) {
 		log_file_error(session, session->file_on_disk, "can't change file mode (PUT)");
 	}
 
+	/* Handle upload range
+	 */
 	if (range_found) {
 		if (strncmp(range, "bytes ", 6) != 0) {
 			result = 416;
@@ -1412,9 +1432,11 @@ int handle_put_request(t_session *session) {
 		}
 	}
 
+	/* Open temporary file for reading
+	 */
 	if ((result == 201) || (result == 204)) {
 		if ((handle_read = open(session->uploaded_file, O_RDONLY)) == -1) {
-			fcntl(handle_write, F_SETLK, F_UNLCK);
+			fcntl(handle_write, F_SETLK, &file_lock);
 			close(handle_write);
 			if (file_size == NEW_FILE) {
 				unlink(session->file_on_disk);
@@ -1426,6 +1448,8 @@ int handle_put_request(t_session *session) {
 			ftruncate(handle_write, session->uploaded_size);
 		}
 		
+		/* Write content
+		 */
 		if ((buffer = (char*)malloc(FILE_BUFFER_SIZE)) != NULL) {
 			while (total_written < session->uploaded_size) {
 				if ((bytes_read = read(handle_read, buffer, FILE_BUFFER_SIZE)) != -1) {
@@ -1448,10 +1472,12 @@ int handle_put_request(t_session *session) {
 		}
 	}
 
+	/* Finish upload
+	 */
 	if (handle_read != -1) {
 		close(handle_read);
 	}
-	fcntl(handle_write, F_SETLK, F_UNLCK);
+	fcntl(handle_write, F_SETLK, &file_lock);
 	fsync(handle_write);
 	close(handle_write);
 	if ((result != 201) && (result != 204) && (file_size == NEW_FILE)) {
@@ -1462,6 +1488,8 @@ int handle_put_request(t_session *session) {
 }
 
 int handle_delete_request(t_session *session) {
+	/* Access check
+	 */
 	switch (allow_alter(session)) {
 		case deny:
 		case unknown:
@@ -1470,7 +1498,7 @@ int handle_delete_request(t_session *session) {
 		case allow:
 			break;
 		case pwd:
-			if (http_authentication_oke(session) == false) {
+			if (http_authentication_oke(session, false) == false) {
 				return 401;
 			}
 			if (group_oke(session, session->remote_user, &(session->host->alter_group)) == false) {
@@ -1479,10 +1507,14 @@ int handle_delete_request(t_session *session) {
 			break;
 	}
 
+	/* Don't delete directories
+	 */
 	if (session->uri_is_dir) {
 		return 405;
 	}
 
+	/* Delete file
+	 */
 	if (unlink(session->file_on_disk) == -1) {
 		log_error(session, fb_filesystem);
 		return 403;
