@@ -11,5 +11,7 @@ int execute_cgi(t_session *session);
 int show_index(t_session *session);
 int handle_options_request(t_session *session);
 int handle_trace_request(t_session *session);
+int handle_put_request(t_session *session);
+int handle_delete_request(t_session *session);
 
 #endif
