@@ -4,6 +4,7 @@
 #include <stdbool.h>
 #include <unistd.h>
 #include <string.h>
+#include "global.h"
 #include "alternative.h"
 #include "libstr.h"
 #include "libfs.h"
@@ -11,7 +12,7 @@
 #include "rewrite.h"
 #endif
 
-#define MAX_INPUT 1024
+#define MAX_INPUT KILOBYTE
 
 typedef struct type_line {
 	char *key, *value, *file;
